#
# Recipe:: ds_opencv
#
# Copyright (c) 2017 The Dark Sky Company, LLC, All Rights Reserved.

# include_recipe 'apt'
# include_recipe 'build-essential'
apt_update
build_essential
include_recipe 'git'

s3_binary_directory = node['ds_opencv']['opencv']['s3_binary_directory']
darksky_ops_available_debs = {
  'x86_64' => {
    '18.04' => {
      '4.3.0' => [
        'OpenCV-4.3.0-x86_64-dev.deb',
        'OpenCV-4.3.0-x86_64-libs.deb',
        'OpenCV-4.3.0-x86_64-licenses.deb',
        'OpenCV-4.3.0-x86_64-python.deb',
        'OpenCV-4.3.0-x86_64-scripts.deb',
      ],
    },
  },
}

# What apt packages to install based on OpenCV major version
packages = {
  '2' => %w(
    cmake
    libgtk2.0-dev
    pkg-config
    libavcodec-dev
    libavformat-dev
    libswscale-dev
    python-dev
    python-numpy
  ),
  '3' => %w(
    cmake
    gfortran
    libjpeg8-dev
    libtiff5-dev
    libpng-dev
    libatlas-base-dev
  ),
  '4' => %w(
    cmake
    gcc
    g++
    python3-dev
    python3-numpy
    libavcodec-dev
    libavformat-dev
    libswscale-dev
  ),
}

opencv_version = node['ds_opencv']['opencv']['version']
opencv_major_version = opencv_version[0]

packages[opencv_major_version].each do |pkg|
  package pkg
end

arch = node[:kernel][:machine]
ubuntu_version = node[:lsb][:release]

# If a binary is available for this OS/arch:
if darksky_ops_available_debs.dig arch, ubuntu_version, opencv_version

  darksky_ops_available_debs[arch][ubuntu_version][opencv_version].each do |deb_name|
    remote_path = File.join s3_binary_directory, "#{opencv_version}-#{arch}-#{ubuntu_version}", deb_name
    local_path = "/tmp/#{deb_name}"

    # Download the deb
    remote_file local_path do
      source remote_path
      backup false
    end

    # Install the deb
    dpkg_package deb_name do
      source local_path
    end
  end

  # Run ldconfig when we're all done
  execute 'ldconfig' do
    command 'ldconfig'
  end

else # Binary not available for this OS/arch
  opencv_path = node['ds_opencv']['opencv']['path']

  git opencv_path do
    repository 'https://github.com/opencv/opencv.git'
    revision   node['ds_opencv']['opencv']['version']
    action     :checkout
  end

  directory "#{opencv_path}/release"

  cmake_define = node['ds_opencv']['opencv']['cmake_define']
                 .map { |k, v| "-D #{k}=#{v}" }.join(' ')

  execute 'cmake_opencv' do
    command "cmake #{cmake_define} .."
    cwd     "#{opencv_path}/release"
    creates "#{opencv_path}/release/Makefile"
  end

  execute 'make_opencv' do
    command 'make -j4 && make install && ldconfig'
    cwd     "#{opencv_path}/release"
    creates node['ds_opencv']['opencv']['cmake_define']['MAKE_INSTALL_PREFIX'] +
            "/lib/libopencv_core.so.#{node['ds_opencv']['opencv']['version']}"
  end
end
